/* Programul asigura filtrarea semnalului de intrare din fisierul input.dat
 * utilizand transformata wavelet 1D implementata in biblioteca GSL. Sunt
 * utilizate pentru filtrare cele mai mari nc componente wavelet ale semnalului
 * initial, toate celelalte fiind anulate */
#include <stdio.h>
#include <math.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_wavelet.h>

int main ()
{
  /* n este numarul de date, nc este numarul de coeficienti wavelet
   * utilizati in procedura de filtrare a semnalului - numai cele
   * mai mari nc valori absolute ale coeficientilor wavelet sunt 
   * retinute, ceilalti coeficienti sunt redusi la 0
   */  
  int i, n = 256, nc = 20;
  double dummy;     // variabila de lucru
  double *data = malloc (n * sizeof (double));  // spatiul de stocare al datelor
  
  // spatiul de stocare al valorilor absolute al coeficientilor wavelet
  double *abscoeff = malloc (n * sizeof (double)); 
  /* spatiul de stocare pentru indicii transformatei wavelet
   * ordonata crescator in valoare absoluta
   */
  size_t *p = malloc (n * sizeof (size_t));

  FILE * f;
  
  // variabile de lucru GSL
  gsl_wavelet *w;
  gsl_wavelet_workspace *work;
  // aloc spatiu de lucru pentru transformata wavelet
  w = gsl_wavelet_alloc (gsl_wavelet_daubechies, 4);
  work = gsl_wavelet_workspace_alloc (n);
  
  // fisierul de date de intrare
  f = fopen ("input.dat", "r");

  for (i = 0; i < n; i++)
   {
     fscanf (f, "%lg\t%lg", &dummy,&data[i]);
   }
   
  fclose (f);
   
  // transformata wavelet directa
  gsl_wavelet_transform_forward (w, data, 1, n, work);
   
  // calculez valorile absolute aale coeficientilor wavelet
  for (i = 0; i < n; i++)
    {
      abscoeff[i] = fabs (data[i]);
    }
    
   /* sortez in ordine crescatoare cele n elemente ale vectorului abscoeff
    * cu deplasarea 1, permutarea rezultata a indicilor elementelor vectorului 
    * fiind stocata in p
    */
  gsl_sort_index (p, abscoeff, 1, n);
   
   /* anulez coeficientii wavelet, cu exceptia primelor cele mai mari
    * nc valori absolute
    */
  for (i = 0; (i + nc) < n; i++)
     data[p[i]] = 0;

   // transformata wavelet inversa
  gsl_wavelet_transform_inverse (w, data, 1, n, work);

   // fisierul de iesire
  f = fopen ("wavelet_signal.dat", "w");
  for (i = 0; i < n; i++)
    {
      fprintf (f,"%lg\t%lg\n", (double) i, data[i]);
    }
  fclose(f);

   // eliberez spatiile RAM alocate
  gsl_wavelet_free (w);
  gsl_wavelet_workspace_free (work);
  free (data);
  free (abscoeff);
  free (p);
  return 0;
}

